package InsulinSignaling "A package for modeling insuling signaling in fat cells"
  package Components "Components used in the main model"
    extends Modelica.Icons.Package;

    model IR_feedback
      extends BioChem.Interfaces.Reactions.Uui;
      extends BioChem.Interfaces.Reactions.Modifiers.Activator;
      parameter Real kbasal = 1;
      parameter Real vmax = 1;
    equation
      rr = s1.c * (kbasal + vmax * a1.c / (1 + a1.c));
      annotation(Documentation(info = "", revisions = ""), Diagram(coordinateSystem(extent = {{-170.0, -105.0}, {170.0, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
    end IR_feedback;

    model IRS1_activation
      extends BioChem.Interfaces.Reactions.Uur;
      parameter Real kr = 1;
      parameter Real kf1 = 1;
      parameter Real kf2 = 1;
      BioChem.Interfaces.Nodes.ActivatorConnector a1 annotation(Placement(visible = true, transformation(origin = {30.0, 70.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0), iconTransformation(origin = {30.0, 90.0}, extent = {{-20.0, -20.0}, {20.0, 20.0}}, rotation = 0)));
      BioChem.Interfaces.Nodes.ActivatorConnector a2 annotation(Placement(visible = true, transformation(origin = {-30.0, 70.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0), iconTransformation(origin = {-30.0, 90.0}, extent = {{-20.0, -20.0}, {20.0, 20.0}}, rotation = 0)));
    equation
      rr = s1.c * (kf1 * a1.c + kf2 * a2.c) - kr * p1.c;
      a1.r = 0;
      a2.r = 0;
      annotation(Documentation(info = "", revisions = ""), Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Line(visible = true, origin = {0.0, 3.75}, points = {{0.0, 56.25}, {0.0, 6.25}}, color = {0, 0, 255}, arrow = {Arrow.None, Arrow.Open}, arrowSize = 25)}), Diagram(coordinateSystem(extent = {{-170.0, -105.0}, {170.0, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
    end IRS1_activation;
    annotation(Documentation, Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end Components;

  model InsulinModel "Model of insulin signaling"
    extends BioChem.Compartments.ConstantCompartment(V.start = 1, mainCompartment = true);
    parameter Real insulinConcentration = 100;
    BioChem.Substances.Substance IRinsulin(c.unit = "au", c.stateSelect = StateSelect.always) "Bound Insulin Receptor" annotation(Placement(visible = true, transformation(origin = {-80, 54.87}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
    BioChem.Substances.Substance IRp(c.unit = "au", c.stateSelect = StateSelect.always, c.start = 0.000326) "Phosphorylated Insulin Receptor" annotation(Placement(visible = true, transformation(origin = {-37.816, 60}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
    BioChem.Reactions.MassAction.Irreversible.UniUni.Uui IRphosphorylation(k1 = 4.48) annotation(Placement(visible = true, transformation(origin = {-59.0095, 57.4881}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 6)));
    BioChem.Substances.SignalSubstance insulin(c.unit = "au") "Signal Substance" annotation(Placement(visible = true, transformation(origin = {-136.864, 76.802}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    BioChem.Substances.Substance IR(c.unit = "au", c.start = 100, n.start = 100, c.stateSelect = StateSelect.always) "Unphosphorylated Insulin Receptor" annotation(Placement(visible = true, transformation(origin = {-122.212, 46.626}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
    BioChem.Reactions.MassAction.Reversible.UniUni.Uur basalphosphorylation(k1 = 0.2427355862, k2 = 70074) annotation(Placement(visible = true, transformation(origin = {-75.2375, 25.7005}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    BioChem.Substances.Substance IRpinternalized(c.unit = "au", c.stateSelect = StateSelect.always, c.start = 1.15166) "Phosphorylated Internalized Insulin Receptor" annotation(Placement(visible = true, transformation(origin = {-35.815, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
    BioChem.Substances.Substance IRinternalized(c.unit = "au", c.stateSelect = StateSelect.always, c.start = 0.0000274573117064578) "Unphosphorylated Internalized Insulin Receptor" annotation(Placement(visible = true, transformation(origin = {-107.154, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 0)));
    BioChem.Reactions.MassAction.Irreversible.UniUni.Uui endocytosis(k1 = 3515) annotation(Placement(visible = true, transformation(origin = {-20.0, 20.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -90)));
    BioChem.Reactions.MassAction.Irreversible.UniUni.Uui recycling(k1 = 41769) annotation(Placement(visible = true, transformation(origin = {-136.4583, 14.652}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = -270)));
    InsulinSignaling.Components.IR_feedback IRdephosphorylation(kbasal = 0.034, vmax = 37) annotation(Placement(visible = true, transformation(origin = {-74.5375, -10.0}, extent = {{10.0, 10.0}, {-10.0, -10.0}}, rotation = 0)));
    BioChem.Substances.Substance IRS1p(c.unit = "au", c.stateSelect = StateSelect.always, c.start = 0.7877) "Phosphorylated Insulin Receptor Substrate" annotation(Placement(visible = true, transformation(origin = {6.916, -27.145}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    BioChem.Substances.Substance IRS1(c.unit = "au", c.start = 99, n.start = 100, c.stateSelect = StateSelect.always) "Unphosphorylated Insulin Receptor Substrate" annotation(Placement(visible = true, transformation(origin = {77.674, -27.145}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    BioChem.Substances.Substance X(c.unit = "au", c.start = 99.97, n.start = 100, c.stateSelect = StateSelect.always) annotation(Placement(visible = true, transformation(origin = {4.8283, -68.37}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    BioChem.Substances.Substance Xactive(c.unit = "au", c.stateSelect = StateSelect.always, c.start = 0.0265284) annotation(Placement(visible = true, transformation(origin = {-55.1717, -68.37}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 0)));
    BioChem.Reactions.MassAction.Reversible.UniUni.Uurfa Xactivation(k2 = 0.0812658755, k1 = 0.0000275772) annotation(Placement(visible = true, transformation(origin = {-25.1717, -68.37}, extent = {{10.0, -10.0}, {-10.0, 10.0}}, rotation = 0)));
    InsulinSignaling.Components.IRS1_activation IRS1phosphorylation(kf1 = 0.765, kf2 = 3.32, kr = 149) annotation(Placement(visible = true, transformation(origin = {42.4405, -27.1451}, extent = {{10.0, -10.0}, {-10.0, 10.0}}, rotation = 0)));
    BioChem.Reactions.MassAction.Reversible.UniUni.Uurfa insulinBinding(k1 = 18047, k2 = 36569) annotation(Placement(visible = true, transformation(origin = {-101.7244, 50.0}, extent = {{-10.0, -10.0}, {10.0, 10.0}}, rotation = 11)));
  equation
    connect(recycling.p1, IR.n1) annotation(Line(visible = true, origin = {-132.897, 34.142}, points = {{-3.562, -8.24}, {-3.562, -0.516}, {-3.562, 12.484}, {10.685, 12.484}}, color = {0, 85, 127}, smooth = Smooth.Bezier));
    connect(IR.n1, insulinBinding.s1) annotation(Line(visible = true, origin = {-118.988, 46.932}, points = {{-3.223, -0.307}, {-3.223, -0.307}, {-3.223, -0.307}, {6.221, 0.921}}, color = {0, 85, 0}, smooth = Smooth.Bezier));
    connect(IR.n1, basalphosphorylation.s1) annotation(Line(visible = true, origin = {-115.439, 23.198}, points = {{-6.772, 23.427}, {-6.772, 2.502}, {28.952, 2.502}, {28.952, 2.502}}, color = {0, 85, 0}, smooth = Smooth.Bezier));
    connect(insulin.n1, insulinBinding.aF1) annotation(Line(visible = true, origin = {-114.582, 70.813}, points = {{-22.281, 5.989}, {11.141, 5.989}, {10.95, -10.996}}, color = {0, 85, 0}, pattern = LinePattern.Dash, smooth = Smooth.Bezier));
    connect(insulinBinding.p1, IRinsulin.n1) annotation(Line(visible = true, origin = {-83.56, 53.962}, points = {{-7.121, -1.815}, {3.56, 0.908}, {3.56, 0.908}}, color = {0, 85, 127}, smooth = Smooth.Bezier));
    connect(IRinsulin.n1, IRphosphorylation.s1) annotation(Line(visible = true, origin = {-66.958, 64.299}, points = {{-13.042, -9.43}, {-11.196, -7.048}, {-13.042, -9.399}, {-3.239, -7.987}}, color = {0, 85, 0}, smooth = Smooth.Bezier));
    connect(IRphosphorylation.p1, IRp.n1) annotation(Line(visible = true, origin = {-41.491, 60.139}, points = {{-6.33, -1.475}, {3.675, 0.046}, {3.675, 0.046}, {3.675, -0.139}}, color = {0, 85, 127}, smooth = Smooth.Bezier));
    connect(IRp.n1, endocytosis.s1) annotation(Line(visible = true, origin = {-20.786, 46.811}, points = {{-17.029, 13.189}, {0.786, 13.189}, {0.786, -12.502}, {0.786, -15.561}}, color = {0, 85, 0}, smooth = Smooth.Bezier));
    connect(IRp.n1, IRS1phosphorylation.a2) annotation(Line(visible = true, origin = {10.687, 29.908}, points = {{-48.503, 30.092}, {-32.207, 30.092}, {34.753, 30.092}, {34.753, -48.053}, {34.753, -48.053}}, color = {0, 85, 0}, pattern = LinePattern.Dash, smooth = Smooth.Bezier));
    connect(basalphosphorylation.p1, IRp.n1) annotation(Line(visible = true, origin = {-46.54, 37.134}, points = {{-17.448, -11.433}, {8.724, -11.433}, {8.724, 22.866}}, color = {0, 85, 127}, smooth = Smooth.Bezier));
    connect(endocytosis.p1, IRpinternalized.n1) annotation(Line(visible = true, origin = {-27.907, -5.312}, points = {{7.907, 14.062}, {7.907, -4.688}, {-7.907, -4.688}, {-7.907, -4.688}}, color = {0, 85, 127}, smooth = Smooth.Bezier));
    connect(Xactive.n1, IRdephosphorylation.a1) annotation(Line(visible = true, origin = {-64.8546, -37.3536}, points = {{9.6829, -31.0164}, {9.6829, 4.7895}, {-9.6829, 4.7895}, {-9.6829, 18.3536}}, color = {0, 85, 0}, pattern = LinePattern.Dash, smooth = Smooth.Bezier));
    connect(IRpinternalized.n1, IRdephosphorylation.s1) annotation(Line(visible = true, origin = {-49.551, -10}, points = {{13.736, 0}, {-13.736, 0}}, color = {0, 85, 0}));
    connect(IRdephosphorylation.p1, IRinternalized.n1) annotation(Line(visible = true, origin = {-96.471, -10}, points = {{10.683, 0}, {-10.683, 0}}, color = {0, 85, 127}));
    connect(IRinternalized.n1, recycling.s1) annotation(Line(visible = true, origin = {-126.69, -5.533}, points = {{19.536, -4.467}, {-9.768, -4.467}, {-9.768, 8.935}}, color = {0, 85, 0}, smooth = Smooth.Bezier));
    connect(IRpinternalized.n1, IRS1phosphorylation.a1) annotation(Line(visible = true, origin = {14.355, -12.715}, points = {{-50.17, 2.715}, {25.085, 2.715}, {25.085, -5.43}}, color = {0, 85, 0}, pattern = LinePattern.Dash, smooth = Smooth.Bezier));
    connect(IRS1p.n1, Xactivation.aF1) annotation(Line(visible = true, origin = {-9.128, -49.314}, points = {{16.044, 22.169}, {16.044, -6.056}, {-16.044, -6.056}, {-16.044, -9.056}}, color = {0, 85, 0}, pattern = LinePattern.Dash, smooth = Smooth.Bezier));
    connect(X.n1, Xactivation.s1) annotation(Line(visible = true, origin = {-4.5467, -68.37}, points = {{9.375, 0.0}, {-9.375, 0.0}}, color = {0, 85, 0}));
    connect(Xactivation.p1, Xactive.n1) annotation(Line(visible = true, origin = {-45.7967, -68.37}, points = {{9.375, 0.0}, {-9.375, 0.0}}, color = {0, 85, 127}));
    connect(IRS1phosphorylation.p1, IRS1p.n1) annotation(Line(visible = true, origin = {19.053, -27.145}, points = {{12.137, -0}, {-12.137, 0}}, color = {0, 85, 127}));
    connect(IRS1.n1, IRS1phosphorylation.s1) annotation(Line(visible = true, origin = {65.682, -27.145}, points = {{11.992, 0}, {-11.992, 0}}, color = {0, 85, 0}));
    insulin.c = insulinConcentration;
    annotation(experiment(StartTime = 0.0, StopTime = 30.0, NumberOfIntervals = -1, __Wolfram_Algorithm = "dassl", Tolerance = 0.000001, __Wolfram_SynchronizeWithRealTime = false), __Wolfram(PlotSet(plots = {Plot(name = "InternalizedIRRates", preferred = true, subPlots = {SubPlot(curves = {Curve(x = time, y = endocytosis.rr, legend = "Endocytosis Rate"), Curve(x = time, y = IRdephosphorylation.rr, legend = "Internalized Insulin Receptor (IR) Dephosphorylation Rate"), Curve(x = time, y = recycling.rr, legend = "Recycling Rate"), Curve(x = time, y = IRS1phosphorylation.rr, legend = "Insulin Receptor Substrate (IRS) Phosphorylation Rate")})}), Plot(name = "IRS1Concentrations", subPlots = {SubPlot(curves = {Curve(x = time, y = IRS1p.c, legend = "Phosphorylated Insulin Receptor Substrate (IRS1p)"), Curve(x = time, y = IRS1.c, legend = "Unphosphorylated Insulin Receptor Substrate (IRS1)")})}), Plot(name = "InternalizedIRConcentrations", subPlots = {SubPlot(curves = {Curve(x = time, y = IRinternalized.c, legend = "Phosphorylated Internalized Insulin Receptor (IRp)"), Curve(x = time, y = IRpinternalized.c, legend = "Unphosphorylated Internalized Insulin Receptor (IR)")})}), Plot(name = "IRConcentrations", subPlots = {SubPlot(curves = {Curve(x = time, y = IRp.c, legend = "Phosphorylated Insulin Receptor (IRp)"), Curve(x = time, y = IR.c, legend = "Unphosphorylated Insulin Receptor (IR)"), Curve(x = time, y = IRinsulin.c, legend = "Bound Insulin Receptor (IR-Insulin)"), Curve(x = time, y = insulin.c, legend = "Insulin")})}), Plot(name = "PhosphorylationRates", subPlots = {SubPlot(curves = {Curve(x = time, y = basalphosphorylation.rr, legend = "Basal Phosphorylation"), Curve(x = time, y = insulinBinding.rr, legend = "Insulin Binding"), Curve(x = time, y = IRphosphorylation.rr, legend = "Bound Phosphyralation")}, range = Range(xmin = 0.126982, xmax = 30.0298, ymin = -1275.1, ymax = 1224.9))})})), preferredView = "info", Documentation(info = "<html>
<div style=\"background-color: #f4fbff; padding: 1rem; border: 2px solid #d9e5f3\">
<h2>Library Dependency</h2>
    <p class=\"\">
		This model requires the <a href=\"https://www.wolfram.com/system-modeler/libraries/biochem/\">BioChem</a> library.
	</p>
	<ul>
		<li style=\"background: url('modelica://EducationExamples/Resources/BioChem.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; \">The free BioChem library is an extendable, general purpose Modelica library for modeling, simulation and visualization of biological and biochemical systems. The library is designed to be used together with Wolfram System Modeler, which enables several extra features such as Systems Biology Markup Language (SBML) import and export. BioChem can, for instance, be used for selecting drug targets with PK/PD modeling or searching for novel drug targets with mechanistic modeling of the reactions in a cell or organism.</li>

	</ul>

</div>


    <h4>Analysis</h4>
    <p>
Simulate the system's response to an insulin stimulation, with a number of different parameter sets found to be plausible:
</p><p>
<img src=\"modelica://InsulinSignaling/DocumentationFiles/compSim.png\" alt=\"compSim\"/>
</p><p>
Plot the sensitivity bounds for parameters in the system, showing how much the IRS1 phosphorylation is affected by changes in each parameter. Find the parameters with the largest influence:
</p><p>
<img src=\"modelica://InsulinSignaling/DocumentationFiles/sensitivity.png\" alt=\"sensitivity\"/>
</p>

<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>

<p>For the full example, open the accompanying notebook <a href=\"modelica://InsulinSignaling/InsulinSignaling.nb\">InsulinSignaling.nb</a>.</p>
</html>"), Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), Diagram(coordinateSystem(extent = {{-170.0, -110.0}, {110.0, 110.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Polygon(visible = true, origin = {-2.0496, 84.1949}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{-167.95, 25.805}, {-167.95, 25.805}, {112.0496, 25.805}, {112.0496, 25.8051}, {112.0496, -24.1949}, {112.0496, -24.1949}, {-0.6329, -4.1949}, {-167.95, -54.195}, {-167.95, -54.195}}, smooth = Smooth.Bezier), Polygon(visible = true, origin = {-0.0, -27.7778}, fillColor = {233, 247, 255}, fillPattern = FillPattern.Solid, points = {{-170.0, -82.222}, {110.0, -82.222}, {110.0, -82.2222}, {110.0, 67.7778}, {110.0, 67.7778}, {0.0, 97.778}, {-170.0, 57.778}, {-170.0, 57.778}, {-170.0, -82.222}}, smooth = Smooth.Bezier), Polygon(visible = true, origin = {-20.837, 55.055}, fillColor = {192, 192, 192}, fillPattern = FillPattern.Solid, points = {{-149.163, -5.055}, {-149.163, -5.055}, {-149.163, -25.055}, {-149.163, -25.055}, {20.837, 4.945}, {130.837, -15.055}, {130.837, -15.055}, {130.837, 4.945}, {130.837, 4.945}, {20.837, 24.945}}, smooth = Smooth.Bezier)}));
  end InsulinModel;
  annotation(preferredView = "info", uses(Modelica(version = "4.0.0"), BioChem(version = "2.0")), __Wolfram(openClass = InsulinSignaling.InsulinModel, Units(symbols = {BaseUnit(symbol = "au", name = "arbitrary unit", presentation = "a.u.", description = "arbitrary unit", unitSystems = {"universal"})})), Documentation(info = "<html>
<div style=\"background-color: #f4fbff; padding: 1rem; border: 2px solid #d9e5f3\">
<h2>Library Dependency</h2>
    <p class=\"\">
		This model requires the <a href=\"https://www.wolfram.com/system-modeler/libraries/biochem/\">BioChem</a> library.
	</p>
	<ul>
		<li style=\"background: url('modelica://EducationExamples/Resources/BioChem.png') no-repeat left top; background-size: 32px 32px; padding-left: 40px; \">The free BioChem library is an extendable, general purpose Modelica library for modeling, simulation and visualization of biological and biochemical systems. The library is designed to be used together with Wolfram System Modeler, which enables several extra features such as Systems Biology Markup Language (SBML) import and export. BioChem can, for instance, be used for selecting drug targets with PK/PD modeling or searching for novel drug targets with mechanistic modeling of the reactions in a cell or organism.</li>

	</ul>

</div>
<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>

<p>For the full example, open the accompanying notebook <a href=\"modelica://InsulinSignaling/InsulinSignaling.nb\">InsulinSignaling.nb</a>.</p>
</html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}));
end InsulinSignaling;
